//+------------------------------------------------------------------+
//|                                           EGEN Trend Advisor.mq4 |
//|                             Copyright  2009, EGEN Software LTD. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+

#property copyright "Copyright  2009, EGEN Software LTD."
#property link      "http://www.metaquotes.net"

#define  OP_ALL   6

extern   bool     CloseAll          =  false;
extern   double   TakeProfit        =  0;
extern   double   StopLoss          =  0;
extern   double   Lots              =  0.01;
extern   bool     Scalper           =  false;
extern   bool     Crazy             =  false;
extern   bool     Impatient         =  false;
extern   int      Delta             =  0;
extern   int      MaxOrders         =  100;
extern   int      Slippage          =  3;

datetime TIME; int LEVEL, TICKET[6]; 

bool EQ(int value1, int value2)
{
   return(value1==value2);
}

bool ZERO(int value)
{
   return(EQ(value,0));
}

int SIGN(int value)
{
   if(value<0) 
      return(-1); 
   if(value>0) 
      return(1);
   return(0);
}

int HIGHEST()
{
   return(iHighest(NULL,0,MODE_CLOSE,PERIOD_D1));
}   

int LOWEST()
{
   return(iLowest(NULL,0,MODE_CLOSE,PERIOD_D1));
}   

double CLOSE(int shift)
{
   return(iClose(NULL,0,shift));
}

datetime TIME(int shift=1)
{
   return(iTime(NULL,0,shift));
}

double INFO(int type)
{
   return(MarketInfo(Symbol(),type));
}

double STOPLEVEL()
{
   return(INFO(MODE_STOPLEVEL)*Point);
}

double SPREAD()
{
   return(INFO(MODE_SPREAD)*Point);
}

double LEVEL(int factor)
{
   return(NormalizeDouble(STOPLEVEL()*FIBO(factor),Digits));
}

double ASK(int factor=0)
{
   return(Ask+LEVEL(factor));
}

double BID(int factor=0)
{
   return(Bid-LEVEL(factor));
}

bool TOTAL(int index)
{
   return(index<OrdersTotal());
}

bool SYMBOL()
{
   return(EQ(OrderSymbol(),Symbol()));
}

bool SelectOrder(int index) 
{
   if(OrderSelect(index,SELECT_BY_POS)) 
      return(SYMBOL());
   return(false);         
}

bool SelectTicket(int ticket) 
{
   if(OrderSelect(ticket,SELECT_BY_TICKET)) 
      return(SYMBOL());
   return(false);         
}

bool CheckTime()
{  
   if(TIME()>TIME)
      return(OrdersTotal()<MaxOrders); 
   return(false);      
}

void BuyLimit()
{  
   if(CheckTime())
      int ticket=OrderSend(Symbol(),OP_BUYLIMIT,Lots,BID(),Slippage,0,0,NULL,0,0,Blue);
   else return;
   if(ticket<0) return; else LEVEL++;
   TIME=TIME(); 
}

void SellLimit()
{
   if(CheckTime())
      int ticket=OrderSend(Symbol(),OP_SELLLIMIT,Lots,ASK(),Slippage,0,0,NULL,0,0,Red);   
   else return;   
   if(ticket<0) return; else LEVEL++;                              
   TIME=TIME();
}

void BuyStop()
{ 
   if(CheckTime())
      int ticket=OrderSend(Symbol(),OP_BUYSTOP,Lots,ASK(),Slippage,0,0,NULL,0,0,Blue); 
   else return;      
   if(ticket<0) return; else LEVEL++;                         
   TIME=TIME(); 
}

void SellStop()
{
   if(CheckTime())
      int ticket=OrderSend(Symbol(),OP_SELLSTOP,Lots,BID(),Slippage,0,0,NULL,0,0,Red);   
   else return;                              
   if(ticket<0) return; else LEVEL++;   
   TIME=TIME();
}

void ClearTicket()
{
   for(int op=OP_BUY; op<OP_ALL; op++) TICKET[op]=EMPTY;
}

void CheckTicket(int shift=0)
{
   ClearTicket();
   for(int i=shift; TOTAL(i); i++)
      if(SelectOrder(i))
         TICKET[OrderType()]=OrderTicket();
      else continue;
}

bool DELTA(int H, int L)
{
   double maxLevel=CLOSE(H)-STOPLEVEL();   
   if(maxLevel<ASK(LEVEL+3)) return(true);   
   double minLevel=CLOSE(L)+STOPLEVEL();      
   if(minLevel>BID(LEVEL+3)) return(true);
   if(Impatient) return(false);
   double delta=SPREAD();
   if(Delta>0) delta=Delta*Point;   
   double level=(minLevel+maxLevel)/2;
   double price=(Ask+Bid)/2;
   if(MathAbs(level-price)>delta) return(true);
   return(false);
}

int TREND()
{
   static int TREND;
   if(LEVEL%4>0) return(TREND);
   int H=HIGHEST(), L=LOWEST();   
   if(DELTA(H,L)) return(0);
   int trend=SIGN(H-L);               
   if(ZERO(TREND+trend)) LEVEL=0;
   if(ZERO(LEVEL)) TREND=trend;
   return(TREND);
}

bool SCALPER()
{ 
   if(Scalper)
   if(AccountProfit()>0)
   if(RemoveAll()) 
      LEVEL=0; else return(true);        
   return(false);         
}

void CheckCrazy()
{
   if(Crazy) 
      if(LEVEL>7) LEVEL=0;
   else return;      
}

void SendOrder() 
{
   if(SCALPER()) return;
   if(TREND()<0) Sell(); else if(TREND()>0) Buy();      
   CheckCrazy();      
}              

double FIBO(int factor)
{
   double FIBO[7]={0.00,1.46,2.36,3.82,5.00,6.18,7.64,8.54};
   int L=factor;
   if(ZERO(L)) L=LEVEL;
   if(ZERO(L)) return(1);   
   double F=FIBO[L%8]+(L/8)*10;
   return(F);
}

void SEND(int op)
{
   if(EQ(op,OP_BUYLIMIT)) BuyLimit(); 
      else
   if(EQ(op,OP_SELLLIMIT)) SellLimit(); 
      else
   if(EQ(op,OP_BUYSTOP)) BuyStop(); 
      else
   if(EQ(op,OP_SELLSTOP)) SellStop();     
}

void Buy()
{ 
   int OP[4]={OP_BUYSTOP,OP_SELLSTOP,OP_BUYLIMIT,OP_SELLLIMIT};
   int op=OP[LEVEL%4];  
   SEND(op);
}
 
void Sell()
{
   int OP[4]={OP_SELLSTOP,OP_BUYSTOP,OP_SELLLIMIT,OP_BUYLIMIT};
   int op=OP[LEVEL%4];  
   SEND(op);
}

int ORDER()
{   
   if(EQ(OrderType(),OP_SELL))       
      return(-1); 
   if(EQ(OrderType(),OP_BUY)) 
      return(1); 
   return(0);      
}  

double PRICE()
{
   if(ORDER()>0) return(Bid);
      else 
   if(ORDER()<0) return(Ask);     
}

void CloseTicket()
{
   if(TIME()>TIME)     
      OrderClose(OrderTicket(),OrderLots(),PRICE(),Slippage,White);           
   else return;         
   TIME=TIME(); 
}

void CloseHedge()
{
   if(TIME()>TIME) 
      OrderCloseBy(BUY(),SELL(),White);           
   else return;      
   TIME=TIME();        
}

void DeleteTicket()
{
   if(TIME()>TIME)
      OrderDelete(OrderTicket(),White); 
   else return;
   TIME=TIME();         
}

int BUY()
{
   return(TICKET[OP_BUY]);
}

int SELL()
{
   return(TICKET[OP_SELL]);
}

int TICKET()
{
   int ticket=EMPTY;
   for(int op=OP_BUYLIMIT; op<OP_ALL; op++)
      if(ticket<TICKET[op]) 
         ticket=TICKET[op]; 
      else continue;         
   return(ticket);
}   

bool RemoveHedge()
{
   if(SelectTicket(BUY()))
      double price1=OrderOpenPrice();
   else return(false);      
   if(SelectTicket(SELL()))
      double price2=OrderOpenPrice();
   else return(false);      
   if(price2<price1) return(false);
   CloseHedge();
   return(true);         
}

bool DeleteOrder()
{ 
   bool flag=Scalper;
   if(CloseAll) flag=false;
   if(flag) return(false);
   if(SelectTicket(TICKET())) 
      DeleteTicket(); else return(false);
   return(true);
}

bool RemoveOrder(int index)
{
   CheckTicket(index);
   if(DeleteOrder()) return(true);
   if(RemoveHedge()) return(true);
   return(false);   
}

bool RemoveAll()
{
   for(int i=0; TOTAL(i); i++) 
      if(RemoveOrder(i)) return(false);
   return(true);      
}

void CloseOrder(int index)
{  
   if(SelectOrder(index)) CloseTicket();
}

void CloseAll()
{
   if(RemoveAll())
      for(int i=0; TOTAL(i); i++) CloseOrder(i);
   else return;      
}

bool TakeProfit()
{
   if(TakeProfit>0)
      return(AccountEquity()>TakeProfit); 
   return(false);      
}

bool StopLoss()
{
   if(StopLoss>0)
      return(AccountEquity()<StopLoss);
   return(false);      
}
  
void CheckTrade()
{
   if(CloseAll) 
      CloseAll(); else
   if(TakeProfit()) 
      CloseAll=true; else
   if(StopLoss()) 
      CloseAll=true; else 
   SendOrder();  
}
  
void start()
{
   if(IsTradeAllowed()) CheckTrade();
}


